package com.nemrodsoftware;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.List;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import com.nemrodsoftware.exception.OperationException;
import com.nemrodsoftware.model.Locale;
import com.nemrodsoftware.model.Module;
import com.nemrodsoftware.model.PermissionEnum;
import com.nemrodsoftware.model.Project;
import com.nemrodsoftware.model.StatusMessage;
import com.nemrodsoftware.model.User;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.GenericType;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.json.JSONConfiguration;
import com.sun.jersey.client.apache.ApacheHttpClient;
import com.sun.jersey.client.apache.config.ApacheHttpClientConfig;
import com.sun.jersey.client.apache.config.DefaultApacheHttpClientConfig;
import com.sun.jersey.core.header.ContentDisposition;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.sun.jersey.multipart.FormDataMultiPart;
import com.sun.jersey.multipart.file.FileDataBodyPart;

public class NemrodClient implements INemrodClient
{
    private final String url;
    private final String apikey;
    
    private final Client client ;
    private WebResource webResource;
    
    public NemrodClient(String url,String apikey)
    {
        this.url = url;
        this.apikey = apikey;
        
        ClientConfig cc = new DefaultApacheHttpClientConfig();
        cc.getFeatures().put(JSONConfiguration.FEATURE_POJO_MAPPING, Boolean.TRUE);
        cc.getProperties().put(ApacheHttpClientConfig.PROPERTY_HANDLE_COOKIES, Boolean.TRUE);
        client = ApacheHttpClient.create(cc);
        client.setFollowRedirects(true);
        webResource = client.resource(this.url);
    }
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#listLocale(com.nemrodsoftware.model.Project)
     */
    @Override
    public List<Locale>listLocale(String project)
    {
        GenericType<List<Locale>> genericType = new GenericType<List<Locale>>() {};
        
        List<Locale> locales = webResource.queryParam("apikey", apikey)
                   .path("/api/locales/"+project)
                   .accept(MediaType.APPLICATION_JSON)
                   .get(genericType);
        return locales;
    }
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#listModule(com.nemrodsoftware.model.Project)
     */
    @Override
    public List<Module>listModule(String project)
    {
        GenericType<List<Module>> genericType = new GenericType<List<Module>>() {};
        
        List<Module> modules = webResource.queryParam("apikey", apikey)
                   .path("/api/modules/"+project)
                   .accept(MediaType.APPLICATION_JSON)
                   .get(genericType);
        return modules;
    }   
    
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#findModule(com.nemrodsoftware.model.Project, java.lang.String)
     */
    @Override
    public Module findModule(String project, String moduleName, String modulePath)
    {
        List<Module> modules = listModule(project);
        for (Module module : modules)
        {
            if (module.getName().equals(moduleName) && module.getFullpath().equals(modulePath))
                return module;
        }
        return null;
    }     
    

    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#viewProject(java.lang.String)
     */
    @Override
    public Project viewProject(String projectName) throws OperationException {
        Project project = webResource.queryParam("apikey", apikey)
                .path("/api/project/"+projectName)
                .accept(MediaType.APPLICATION_JSON)
                .get(Project.class);
        return project;
    }
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#export(com.nemrodsoftware.model.Project, com.nemrodsoftware.model.Locale, com.nemrodsoftware.model.Module, java.lang.String, boolean, boolean)
     */
    @Override
    public File export(String project, String localeName, Module module, String exportType, boolean exportEmptyValues, boolean exportOnlyApprovedValues, String workFolder) throws IOException, OperationException
    {
        MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();
        queryParams.add("exportemptyvalues", Boolean.toString(exportEmptyValues));
        queryParams.add("onlyapprovedvalues", Boolean.toString(exportOnlyApprovedValues));
        queryParams.add("apikey", apikey);
        
        ClientResponse response = webResource.queryParams(queryParams)
                    .path("/api/translations/"+project+"/"+exportType+"/"+localeName+"/"+module.getId())
                    .accept(MediaType.APPLICATION_OCTET_STREAM)
                    .get(ClientResponse.class);
        
        checkStatusMessage(response);
        
        String header = response.getHeaders().getFirst("Content-Disposition");
        ContentDisposition cd;
        try
        {
            cd = new ContentDisposition(header);
        } catch (ParseException e)
        {
            throw new IllegalStateException("header Content-disposition not found or invalid. Header found="+header,e);
        }
        String filename = cd.getFileName();
        InputStream is = response.getEntityInputStream();
        
        File dest = new File (workFolder,filename);
        
        FileUtils.writeByteArrayToFile(dest, IOUtils.toByteArray(is));
        
        return dest;        
    }

    @Override
    public void sync(String project, String format, Module module, File masterFile) throws OperationException
    {
        FormDataMultiPart form = new FormDataMultiPart();
        form.bodyPart(new FileDataBodyPart("masterFile", masterFile,  MediaType.APPLICATION_OCTET_STREAM_TYPE));
        
        ClientResponse response = webResource.queryParam("apikey", apikey)
        .path("/api/sync/"+project + "/"+format+"/"+module.getId())
        .type(MediaType.MULTIPART_FORM_DATA_TYPE)
        .accept(MediaType.APPLICATION_JSON)
        .post(ClientResponse.class,form);
        
        checkStatusMessage(response);
    }

    @Override
    public void create(Project project) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
        .queryParam("project.name", project.getName())
        .queryParam("project.description", project.getDescription())
        .queryParam("project.isPublic", project.getIsPublic().toString())
        .queryParam("project.nbMaxLocale", project.getNbMaxLocale().toString())
        .queryParam("project.nbMaxUser", project.getNbMaxUser().toString())
        .queryParam("project.nbMaxMessage", project.getNbMaxMessage().toString())
        .path("/api/project")
        .accept(MediaType.APPLICATION_JSON)
        .post(ClientResponse.class);
        
        checkStatusMessage(response);
    }

    

    @Override
    public void delete(String projectName) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
        .path("/api/project/"+projectName)
        .accept(MediaType.APPLICATION_JSON)
        .delete(ClientResponse.class);
        
        checkStatusMessage(response);        
    }

    @Override
    public void setVisibility(String projectName, Boolean isPublic) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
                                             .queryParam("isPublic", isPublic.toString())
                                            .path("/api/conf/visibility/"+projectName)
                                            .accept(MediaType.APPLICATION_JSON)
                                            .post(ClientResponse.class);
        
        checkStatusMessage(response);  
    }

    @Override
    public void setNbMaxUser(String projectName, Long nbMaxUser) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
                                            .queryParam("nbMaxUser", nbMaxUser.toString())
                                           .path("/api/conf/users/"+projectName)
                                           .accept(MediaType.APPLICATION_JSON)
                                           .post(ClientResponse.class);

        checkStatusMessage(response);  
        
    }

    @Override
    public void setNbMaxLocale(String projectName, Long nbMaxLocale) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
                                    .queryParam("nbMaxLocale", nbMaxLocale.toString())
                                   .path("/api/conf/locales/"+projectName)
                                   .accept(MediaType.APPLICATION_JSON)
                                   .post(ClientResponse.class);

        checkStatusMessage(response);  
        
    }

    @Override
    public void setNbMaxMessage(String projectName, Long nbMaxMessage) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
                                            .queryParam("nbMaxMessage", nbMaxMessage.toString())
                                           .path("/api/conf/messages/"+projectName)
                                           .accept(MediaType.APPLICATION_JSON)
                                           .post(ClientResponse.class);

        checkStatusMessage(response);  
        
    }

    @Override
    public Long getNbMaxMessage(String projectName) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
       .path("/api/conf/messages/"+projectName)
       .accept(MediaType.APPLICATION_JSON)
       .get(ClientResponse.class);

        checkStatusMessage(response);
        
        StatusMessage msg = response.getEntity(StatusMessage.class);
        
        return Long.valueOf(msg.getDetail());
    }

    @Override
    public Long getNbMaxLocale(String projectName) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
        .path("/api/conf/locales/"+projectName)
        .accept(MediaType.APPLICATION_JSON)
        .get(ClientResponse.class);

        checkStatusMessage(response);
         
         StatusMessage msg = response.getEntity(StatusMessage.class);
         
         return Long.valueOf(msg.getDetail());
    }

    @Override
    public Long getNbMaxUser(String projectName) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
        .path("/api/conf/users/"+projectName)
        .accept(MediaType.APPLICATION_JSON)
        .get(ClientResponse.class);

         checkStatusMessage(response);
         
         StatusMessage msg = response.getEntity(StatusMessage.class);
         
         return Long.valueOf(msg.getDetail());
    }

    @Override
    public void addMember(String projectName, String login) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
                       .path("/api/member/"+projectName+"/"+login)
                       .accept(MediaType.APPLICATION_JSON)
                       .post(ClientResponse.class);

        checkStatusMessage(response);
    }
    

    @Override
    public void createUser(User user) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
        .queryParam("user.fullname", user.getFullname())
        .queryParam("user.login", user.getLogin())
        .queryParam("user.email", user.getEmail())
        .queryParam("user.password", user.getPassword())
        .path("/api/user")
        .accept(MediaType.APPLICATION_JSON)
        .post(ClientResponse.class);

        checkStatusMessage(response);
        
    }

    @Override
    public void allow(String projectName, String login, PermissionEnum permission) throws OperationException
    {
        ClientResponse response = webResource.queryParam("apikey", apikey)
        .queryParam("permission", permission.name())
       .path("/api/allow/"+projectName+"/"+login)
       .accept(MediaType.APPLICATION_JSON)
       .post(ClientResponse.class);

        checkStatusMessage(response);
    }    

    private void checkStatusMessage(ClientResponse response) throws OperationException
    {
        switch(response.getStatus())
        {
            case 400 :
            StatusMessage msg = response.getEntity(StatusMessage.class);
            throw new OperationException(response.getStatus(),msg.getDetail(),msg.getErrors());
            
            case 404 :
                throw new OperationException(response.getStatus(),"Resource not found");
            case 403 :
                throw new OperationException(response.getStatus(),"Forbidden");
            case 500 :
                throw new OperationException(response.getStatus(),"Unexpected error.");
            case 200 :
            default :
        }
    }
}
